﻿using Microsoft.EntityFrameworkCore;
using UserMicroservice.Models;

namespace UserMicroservice.Services
{
    public class UserService : IUserService
    {
        // look at entity framwork documetation for queries to the db using the DbContext
        private readonly ApplicationDbContext _context;
        private readonly IAuthService _authService;

        public UserService(ApplicationDbContext context, IAuthService authService)
        {
            _context = context;
            _authService = authService;
        }

        public bool DeleteUser(string username)
        {
            User user = _context.Users.Single(user => user.Username == username);
            if(user == null)
                return false;

            _context.Users.Remove(user);
            _context.SaveChanges();

            return true;
        }

        public List<User> GetUsers() 
        { 
            List<User> users = _context.Users.ToList();
            return users;
        }

        public User GetUser(string username)
        {
            User user = _context.Users.Single(user => user.Username == username);
            return user;
        }

        public string? Login(string username, string password)
        {
            User user = _context.Users
                .Single(user => user.Username == username && user.PasswordHash == password);

            if (user == null)
                return null;

            return _authService.GenerateToken(user);
        }

        public string RegisterUser(string username, string password)
        {
            User user = new User { Username = username, PasswordHash = password }; // use the contructor later
            _context.Users.Add(user);
            _context.SaveChanges();
            
            return _authService.GenerateToken(user);
        }

        public User UpdateUser(User updatedUser)
        {
            _context.Users
                        .Where(user => user.Id == updatedUser.Id)
                        .ExecuteUpdate(setters => setters
                            .SetProperty(user => user.Username, updatedUser.Username)
                            .SetProperty(user => user.PasswordHash, updatedUser.PasswordHash));

            _context.SaveChanges();

            return updatedUser;
        }
    }
}
